import dataclasses
from typing import Union

import requests


@dataclasses.dataclass
class Flair:
    label: str
    name: str
    description: str
    hidden: bool
    priority: int
    color: str
    rainbowColor: bool
    image: list[dict]

    def __repr__(self):
        return f"Flair<{self.label}>"

    def __hash__(self):
        return hash(self.name)


def flair_converter(endpoint: str) -> dict:
    """Returns a dict to convert flair names (e.g. flair17) to a Flair object."""
    r = requests.get(endpoint)
    converter = {item["name"]: Flair(**item) for item in r.json()}
    return converter


def convert_flairs(flair_dict: dict, features: list = None) -> Union[list[Flair], None]:
    """Converts a list of features into their corresponding Flairs, using a dict created from flair_converter."""
    if features:
        return [flair_dict[flair] for flair in features]



from .bot import DGGBot
from .chat import DGGChat
from .event import EventType
from .flairs import Flair
from .live import DGGLive
from .live.message import Stream, StreamInfo, YoutubeVideo, YoutubeVod
from .message import (
    BroadcastMessage,
    DonationMessage,
    GiftSubMessage,
    MassGiftMessage,
    Message,
    MuteMessage,
    PinnedMessage,
    PollMessage,
    PrivateMessage,
    SubscriptionMessage,
    VoteMessage,
)

VERSION = "1.4.0"



"""
Checks and tests for the following:
 - Connecting to DGG chat
 - (more to come whenever I think of more)
"""

import threading
import time

from dggbot import DGGChat

import pytest

checks = {"connection": False}


class TestChat(DGGChat):
    def _on_open(self, ws):
        super()._on_open(ws)
        checks["connection"] = True


chat = TestChat()


@pytest.mark.order(0)
def test_connection():
    t = threading.Thread(target=chat.run)
    t.start()
    time.sleep(5)
    assert all(v for v in checks.values())





@pytest.mark.order(after="test_connection")
def test_other():
    chat.ws.close()
    assert "A" in "abefeasfAfaefm"
